"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = loginUser;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const encrypt_1 = __importDefault(require("../../functions/dsql/encrypt"));
const grab_host_names_1 = __importDefault(require("../../utils/grab-host-names"));
const api_login_1 = __importDefault(require("../../functions/api/users/api-login"));
const get_auth_cookie_names_1 = __importDefault(require("../../functions/backend/cookies/get-auth-cookie-names"));
const write_auth_files_1 = require("../../functions/backend/auth/write-auth-files");
/**
 * # Login A user
 */
function loginUser(_a) {
    return __awaiter(this, arguments, void 0, function* ({ key, payload, database, additionalFields, response, encryptionKey, encryptionSalt, email_login, email_login_code, temp_code_field, token, user_id, skipPassword, apiUserID, skipWriteAuthFile, dbUserId, debug, }) {
        var _b;
        const grabedHostNames = (0, grab_host_names_1.default)({ userId: user_id || apiUserID });
        const { host, port, scheme } = grabedHostNames;
        const defaultTempLoginFieldName = "temp_login_code";
        const emailLoginTempCodeFieldName = email_login
            ? temp_code_field
                ? temp_code_field
                : defaultTempLoginFieldName
            : undefined;
        const finalEncryptionKey = encryptionKey || process.env.DSQL_ENCRYPTION_PASSWORD;
        const finalEncryptionSalt = encryptionSalt || process.env.DSQL_ENCRYPTION_SALT;
        if (!(finalEncryptionKey === null || finalEncryptionKey === void 0 ? void 0 : finalEncryptionKey.match(/.{8,}/))) {
            console.log("Encryption key is invalid");
            return {
                success: false,
                payload: null,
                msg: "Encryption key is invalid",
            };
        }
        if (!(finalEncryptionSalt === null || finalEncryptionSalt === void 0 ? void 0 : finalEncryptionSalt.match(/.{8,}/))) {
            console.log("Encryption salt is invalid");
            return {
                success: false,
                payload: null,
                msg: "Encryption salt is invalid",
            };
        }
        /**
         * Check required fields
         *
         * @description Check required fields
         */
        if (!payload.email) {
            return {
                success: false,
                payload: null,
                msg: "Email Required",
            };
        }
        /**
         * Initialize HTTP response variable
         */
        /** @type {import("../../types").APILoginFunctionReturn} */
        let httpResponse = {
            success: false,
        };
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } = process.env;
        if ((DSQL_DB_HOST === null || DSQL_DB_HOST === void 0 ? void 0 : DSQL_DB_HOST.match(/./)) &&
            (DSQL_DB_USERNAME === null || DSQL_DB_USERNAME === void 0 ? void 0 : DSQL_DB_USERNAME.match(/./)) &&
            (DSQL_DB_PASSWORD === null || DSQL_DB_PASSWORD === void 0 ? void 0 : DSQL_DB_PASSWORD.match(/./)) &&
            (DSQL_DB_NAME === null || DSQL_DB_NAME === void 0 ? void 0 : DSQL_DB_NAME.match(/./)) &&
            global.DSQL_USE_LOCAL) {
            let dbSchema;
            try {
                const localDbSchemaPath = path_1.default.resolve(process.cwd(), "dsql.schema.json");
                dbSchema = JSON.parse(fs_1.default.readFileSync(localDbSchemaPath, "utf8"));
            }
            catch (error) { }
            httpResponse = yield (0, api_login_1.default)({
                database: process.env.DSQL_DB_NAME || "",
                email: payload.email,
                username: payload.username,
                password: payload.password,
                skipPassword,
                encryptionKey: finalEncryptionKey,
                additionalFields,
                email_login,
                email_login_code,
                email_login_field: emailLoginTempCodeFieldName,
                token,
                dbUserId,
                debug,
            });
        }
        else {
            httpResponse = yield new Promise((resolve, reject) => {
                const reqPayload = {
                    encryptionKey: finalEncryptionKey,
                    payload,
                    database,
                    additionalFields,
                    email_login,
                    email_login_code,
                    email_login_field: emailLoginTempCodeFieldName,
                    token,
                    skipPassword: skipPassword,
                    dbUserId: dbUserId || 0,
                };
                const reqPayloadJSON = JSON.stringify(reqPayload);
                const httpsRequest = scheme.request({
                    method: "POST",
                    headers: {
                        "Content-Type": "application/json",
                        "Content-Length": Buffer.from(reqPayloadJSON).length,
                        Authorization: key ||
                            process.env.DSQL_FULL_ACCESS_API_KEY ||
                            process.env.DSQL_API_KEY,
                    },
                    port,
                    hostname: host,
                    path: `/api/user/${user_id || grabedHostNames.user_id}/login-user`,
                }, (res) => {
                    var str = "";
                    res.on("data", function (chunk) {
                        str += chunk;
                    });
                    res.on("end", function () {
                        resolve(JSON.parse(str));
                    });
                    res.on("error", (err) => {
                        reject(err);
                    });
                });
                httpsRequest.write(reqPayloadJSON);
                httpsRequest.end();
            });
        }
        if (debug) {
            console.log(`loginUser:httpResponse:`, httpResponse);
        }
        if (httpResponse === null || httpResponse === void 0 ? void 0 : httpResponse.success) {
            let encryptedPayload = (0, encrypt_1.default)({
                data: JSON.stringify(httpResponse.payload),
                encryptionKey: finalEncryptionKey,
                encryptionSalt: finalEncryptionSalt,
            });
            try {
                if (token && encryptedPayload)
                    httpResponse["token"] = encryptedPayload;
            }
            catch (error) { }
            const cookieNames = (0, get_auth_cookie_names_1.default)({
                database,
                userId: grabedHostNames.user_id,
            });
            if (httpResponse.csrf && !skipWriteAuthFile) {
                (0, write_auth_files_1.writeAuthFile)(httpResponse.csrf, JSON.stringify(httpResponse.payload));
            }
            httpResponse["cookieNames"] = cookieNames;
            httpResponse["key"] = String(encryptedPayload);
            const authKeyName = cookieNames.keyCookieName;
            const csrfName = cookieNames.csrfCookieName;
            if (debug) {
                console.log(`loginUser:authKeyName:`, authKeyName);
                console.log(`loginUser:csrfName:`, csrfName);
                console.log(`loginUser:encryptedPayload:`, encryptedPayload);
            }
            response === null || response === void 0 ? void 0 : response.setHeader("Set-Cookie", [
                `${authKeyName}=${encryptedPayload};samesite=strict;path=/;HttpOnly=true;Secure=true`,
                `${csrfName}=${(_b = httpResponse.payload) === null || _b === void 0 ? void 0 : _b.csrf_k};samesite=strict;path=/;HttpOnly=true`,
            ]);
            if (debug) {
                console.log(`loginUser:Response Sent!`);
            }
        }
        return httpResponse;
    });
}
